import typescript from 'rollup-plugin-typescript2'
import commonjs from 'rollup-plugin-commonjs'
import external from 'rollup-plugin-peer-deps-external'
import postcss from 'rollup-plugin-postcss'
import resolve from 'rollup-plugin-node-resolve'
import url from 'rollup-plugin-url'
import svgr from '@svgr/rollup'

import pkg from './package.json'

export default {
  input: 'src/index.tsx',
  output: [
    {
      file: pkg.main,
      format: 'cjs',
      exports: 'named',
      sourcemap: true,
      <% if (!!cssinjs) { %>
      globals: { "styled-components": "styled" }
      <% } %>
    },
    {
      file: pkg.module,
      format: 'es',
      exports: 'named',
      sourcemap: true,
      <% if (!!cssinjs) { %>
      globals: { "styled-components": "styled" }
      <% } %>
    },
  ],
  external: ["react", "react-dom", "prop-types", <% if (!!cssinjs) { %>"styled-components"<% } %>],
  plugins: [
    external(),
    postcss({
      modules: true,
    }),
    url(),
    svgr(),
    resolve(),
    typescript({
      rollupCommonJSResolveHack: true,
      clean: true,
    }),
    <% if (!!cssinjs) { %>
    commonjs({
      include: "node_modules/**",
      namedExports: {
        "node_modules/react-is/index.js": ["isValidElementType"]
      }
    }),
    <% } else { %>
    commonjs(),
    <% } %>
  ],
}
